%%%Marissa Tremblay, EPS 109 Fall 2012 final project

%%%This code simulates the simultaneous production and diffusion of
%%%cosmogenic 3He in a grain of quartz for a specified isothermal holding
%%%temperature at the Earth's surface and over a specified period of time.
%%%The code utilizes data I collected on the diffusion kinetics of 3He in
%%%quartz through stepwise heating and degasisng experiments on
%%%proton-irradiated grains.

clear all; close all; clc;

%Simulate a quartz grain with radius 1mm (in cm for later calculations)
grainradius = 0.1; 
T = 0 + 273.15;  %Isothermal holding temperature, in Kelvin

R = .008314;     %gas constant, in kJ/(mol*K)
Ea = 99.0;       %Average activation energy for 3He in quartz deterimined 
                 %through laboratory experiments, in kJ/mol
D0 = exp(8.03); %Pre-exponential factor D0 for 3He in quartz deterimined 
                 %through laboratory experiments, in cm^2/sec

%Normalize pre-exponential factor for grain size, sec^-1
D0a2 = D0./(grainradius.^2); 
%Calculate diffusivity normalized to grain size, sec^-1
Da2 = D0a2.*exp(-Ea./(R.*T)).*(60*60*24*365.25); 

t = linspace(0,50000,20000); %Time, in yr
dt = max(t)./length(t); %Size of time step
time = length(t); %Number of time steps to be carried out

gridspace = 0.105; %Size of the simulation in cm
dgridspace = 0.005; %Spacing in cm
radius = 0.005:dgridspace:gridspace; %Define range of radii
%Define the cartesian coordinates xx and yy for 2D plotting
xx = -gridspace:0.001:gridspace;
yy = -gridspace:0.001:gridspace;

%Initialize the matrices quartz, which represents concentration as a
%function of radial position and time in spherical coordinates, 
%quartz_cart, which represents the concentration in cartesian coordinates, 
%and quartz_no_diffusion, quartz_no_diffusion_cart, which represent only
%production as a function of time for spherical and cartesian coordinates 
quartz = zeros(length(radius),time);
quartz_cart = zeros(length(xx),length(yy));
quartz_no_diffusion = zeros(length(radius),time);
quartz_no_diffusion_cart = zeros(length(xx),length(yy));

Prod_rate_grams = 125; %The production rate of 3He in quartz is ~125 atoms/g*yr
density_quartz = 2.655; %Average density of quartz, g/cm^3
%Calculate production rate for each grid space in atoms/um^3/yr
Prod_rate_grid = Prod_rate_grams.*density_quartz.*(dgridspace^3);

f = 0; %update f each time an image is saved
V = [0 1]; %dimensions of the colorbar in plot

%Solve Fick's Second Law of diffusion for a quartz grain with spherical 
%geometry using a finite difference method.
for i = 1:time
    quartz_new = quartz;
    for k = 1
        quartz_new(k,i+1) = quartz(k,i) + Prod_rate_grid*dt + ...
            (Da2*dt)*(((2*quartz(k+1,i)-2*quartz(k,i))/(dgridspace.^2))...
            +((2./radius(k))*((quartz(k+1,i)-quartz(k,i))/dgridspace)));
    end
    for k = 2:length(radius)-1
        quartz_new(k,i+1) = quartz(k,i) + Prod_rate_grid*dt + ...
            (Da2*dt)*(((quartz(k+1,i)+quartz(k-1,i)-2*quartz(k,i))/...
            (dgridspace.^2))+((2./radius(k))*((quartz(k+1,i)-...
            quartz(k,i))/dgridspace)));
    end
    for k = 1:length(radius)-1
        quartz_no_diffusion(k,i+1) = quartz_no_diffusion(k,i) + ...
            Prod_rate_grid*dt;
    end
    
    quartz = quartz_new;
    quartz_time = quartz_new(:,i);
    quartz_no_diffusion_time = quartz_no_diffusion(:,i);
    
    %Convert the radial concentrations into a 2D concentration gradient in
    %cartesian coordinates
    for p = 1
        for q = 1:length(xx)
            for m = 1:length(yy)
                if (xx(q)^2)+(yy(m)^2) <= radius(p)^2
                    quartz_cart(q,m) = quartz_time(p);
                    quartz_no_diffusion_cart(q,m) = quartz_no_diffusion_time(p);
                end
            end
        end
    end
    for p = 2:length(radius)-1
        for q = 1:length(xx)
            for m = 1:length(yy)
                if (xx(q)^2)+(yy(m)^2) <= radius(p)^2 && (xx(q)^2)+(yy(m)^2) > radius(p-1)^2
                    quartz_cart(q,m) = quartz_time(p);
                    quartz_no_diffusion_cart(q,m) = quartz_no_diffusion_time(p);
                end
            end
        end
    end
    
    %Calculate the ratio of the concentration of quartz 3He relative to the
    %concentration if no diffusion was occuring
    fractional_loss_cart = quartz_cart./quartz_no_diffusion_cart;
    
    [X,Y] = meshgrid(xx,yy);
    
    %Plot
    clf;
    if mod(i+48,50)== 0;
        f = f+1;
        Time = t(i);
        surface(X,Y,fractional_loss_cart,'EdgeColor','none');
        title('Production and diffusion of cosmogenic ^3He in quartz','FontSize',15);
        colorbar;
        caxis(V);
        text(0.03,0.11,[num2str(Time,5),' yr']);
        axis([-0.12 0.12 -0.12 0.12]);
        % axis('equal');
        saveas(gcf, ['Helium_diffusion_cross_section',sprintf('%03d',f),'.png']);
        pause(0.01);
    end
end